{
	// Toms Toolbox v04
	//
	// ©2021-2024 Tom Bloczynski
	// tombloczynski.com
	//
	// This collection of scripts was made to automate some of our more tedious tasks.
	//
var version = "04";
var scriptName = "Tom's Toolbox";
var newWindow;


	function toolbox(thisObj){
		
		
		//This function builds the UI.
		function toolbox_buildUI(thisObj){
			var panel = (thisObj instanceof Panel) ? thisObj : new Window("palette", scriptName, undefined, {resizeable:true, closeButton: true, borderless:true});		
			
			var tabs = panel.add("tabbedpanel");
			tabs.alignment = ["fill", "top"];
			tabs.spacing = 0;
			tabs.alignChildren = ["center", "top"];
			tabs.spacing = 5;
			tabs.resizeable = true;
			
			var tabTidy = tabs.add("tab", undefined, "Tidy Up");
			//var tabPalette = tabs.add("tab", undefined, "Palette");
			//tabPalette.orientation = "column";
			var tabAutonamer = tabs.add("tab", undefined, "Autonamer");
			tabAutonamer.orientation = "column";
			var tabExpression = tabs.add("tab", undefined, "Expressions");
			/*var tabRenamer = tabs.add("tab", undefined, "Renamer");
			tabRenamer.orientation = "column";
			var tabComps = tabs.add("tab", undefined, "Comps");
			tabComps.orientation = "column";*/
			var tabBatch = tabs.add("tab", undefined, "Batch");
			
			var tabAbout = tabs.add("tab", undefined, "About");



			//Tidy Tab
			tabTidy.orientation = "row";
			tabTidy.alignChildren = ["center", "top"];
			tabTidy.spacing = 5;
			
			var dropdown = tabTidy.add("dropdownlist", undefined, "Dropdown");
			dropdown.minimumSize = [150, 20];
			var item01 = dropdown.add ("item", "Remove Unused Effects");
			var item02 = dropdown.add ("item", "Remove Empty Folders");
			var item06 = dropdown.add ("item", "Remove Guide Layers");
			var item07 = dropdown.add ("item", "Hide Viewer Guides");
			var item03 = dropdown.add ("item", "Organize Assets");
			var item04 = dropdown.add ("item", "Gray Everything");
			//var item11 = dropdown.add ("item", "Replace Fonts");
			var item05 = dropdown.add ("item", "Autoname Layers");
			//var item12 = dropdown.add("item", "Batch Renamer");
			//var item13 = dropdown.add("item", "Batch Font Changer");

			
			//var item08 = dropdown.add ("item", "Nudge Keyframes");
			dropdown.selection = dropdown.items[0];

			var runButton = tabTidy.add("button", undefined, "Run");
			runButton.preferredSize = [50, 25];
			dropdown.helpTip = "Searches through the project and removes all disabled effects.";
			runButton.helpTip = "Searches through the project and removes all disabled effects.";
			dropdown.onChange = function() {
				var text = String(dropdown.selection.text);
				
				if(text == "Remove Unused Effects"){
					dropdown.helpTip = "Searches through the project and removes all disabled effects.";
					runButton.helpTip = "Searches through the project and removes all disabled effects.";
				}
				else if (text == "Remove Guide Layers"){
					dropdown.helpTip = "Deletes all guide layers from every comp.";
					runButton.helpTip = "Deletes all guide layers from every comp.";
				}
				else if(text == "Organize Assets"){
					dropdown.helpTip = 'Organizes all assets in the project panel by file extention and puts them in a folder called "Assets".';
					runButton.helpTip = 'Organizes all assets in the project panel by file extention and puts them in a folder called "Assets".';
				}
				else if(text == "Hide Viewer Guides"){
					dropdown.helpTip = "Opens up all comps in the viewer one by one and hides all their rulers and viewer guides.";
					runButton.helpTip = "Opens up all comps in the viewer one by one and disables all their rulers and viewer guides.";		
				}
				else if(text == "Remove Empty Folders"){
					dropdown.helpTip = "Removes all folders in the project panel that don't contain anything.";
					runButton.helpTip = "Removes all folders in the project panel that don't contain anything.";
				}
				else if(text == "Gray Everything"){
					dropdown.helpTip = "Removes the label color from every single item and layer in the project.";
					runButton.helpTip = "Removes the label color from every single item and layer in the project.";
				}
				else if(text == "Autoname Layers"){
					dropdown.helpTip = "Runs through every layer in every comp and names them based on their effects if they are an unnamed adjustment layer or solid.";
					runButton.helpTip = "Runs through every layer in every comp and names them based on their effects if they are an unnamed adjustment layer or solid.";
				}
				else if(text == "Replace Fonts"){
					dropdown.helpTip = "Replaces all the selected fonts in the project with a font of your choice from your active comp.";
					runButton.helpTip = "Replaces all the selected fonts in the project with a font of your choice from your active comp.";
				}
				else if (text == "Nudge Keyframes"){
					dropdown.helpTip = "Nudges all keyframes over so they are on whole frames and aren't between frames.";
					runButton.helpTip = "Nudges all keyframes over so they are on whole frames and aren't between frames.";
				}
			}
			runButton.onClick = function() {
				var text = String(dropdown.selection.text);
				
				if(text == "Remove Unused Effects"){
					removeUnusedEffects();
				}
				else if (text == "Remove Guide Layers"){
					removeGuideLayers();
				}
				else if(text == "Organize Assets"){
					organizeAssets();
				}
				else if(text == "Hide Viewer Guides"){
					turnOffViewerGuides();			
				}
				else if(text == "Remove Empty Folders"){
					removeEmptyFolders();
				}
				else if(text == "Gray Everything"){
					gray();
				}
				else if(text == "Autoname Layers"){
					try{
						autonameLayers();
					}
					catch(err){
						alert(err);
					}

				}
				else if(text == "Nudge Keyframes"){
					nudgeKeyframes();
				}
				else if(text == "Replace Fonts"){
					replaceFonts();
				}
				else if(text == "Batch Renamer"){
					batchCompRenamerWindow();
				}
				else if(text == "Batch Font Changer"){
					batchFontChangerWindow();
				}
				else{
					alert("No script defined", scriptName);
				}
			}
			
			//Palette
			
			
			/**var groupPalette = tabPalette.add("group");
			groupPalette.orientation = "row";
			
			var palette01 = groupPalette.add("button", undefined, "");
			palette01.size = [30,20];
			palette01.fillBrush = palette01.graphics.newBrush(palette01.graphics.BrushType.SOLID_COLOR,[1,0,0]);
			palette01.onDraw = customDraw;
			
			var hex = [1,0,0];
			//var hex = "0xff0000";
			
			palette01.onClick = ninjaColorPicker;
			
			function ninjaColorPicker(){
					var startValue = hex;
					if(!startValue || startValue.length != 3){
						startValue = [1, 1, 1]; // default value
					}

					var comp = app.project.activeItem;
					if(!comp || !(comp instanceof CompItem)){
						alert("Please select a comp before changing palette colors.");
						return null;
					}
					

					// add a temp null;
					
					var newSolid = comp.layers.addSolid(startValue, "Palette01", 100, 100, 1);
					//var newNull = comp.layers.addNull();
					var newColorControl = newSolid.property("ADBE Effect Parade").addProperty("ADBE Color Control");
					var theColorProp = newColorControl.property("ADBE Color Control-0001");

					// shy and turn eyeball off
					var origShyCondition = comp.hideShyLayers;
					if(origShyCondition == false) comp.hideShyLayers = true;
					newSolid.shy = true;
					newSolid.enabled = false;

					// set the value given by the function arguments
					theColorProp.setValue(startValue);

					// prepare to execute
					var editValueID = app.findMenuCommandId("Edit Value...");
					theColorProp.selected = true;
					app.executeCommand(editValueID);

					// harvest the result
					var result = theColorProp.value;
					newSolid.color = [result[0], result[1], result[2]];

					// remove the null
					if(newSolid){
						newSolid.remove();
					}

					// get shy condition back to original
					comp.hideShyLayers = origShyCondition;


					updateColorButton(palette01, result);
					hex = [result[0], result[1], result[2]];
				
				
				
				/*
				var colorPicker = $.colorPicker(hex);
				if(colorPicker != -1){
					hex = colorPicker;
					var r = colorPicker >> 16;
					var g = (colorPicker & 0x00ff00) >> 8;
					var b = colorPicker & 0xff;
					updateColorButton(palette01, [r/255, g/255, b/255]);
				}
				

			}*/
			
			
			/**function customDraw(){
				with(this){
					graphics.drawOSControl();
					graphics.rectPath(0,0,size[0],size[1]);
					graphics.fillPath(fillBrush);
				}
			}
			
			function updateColorButton(button, color){
				button.fillBrush = button.graphics.newBrush(button.graphics.BrushType.SOLID_COLOR,color);
				button.onDraw = customDraw;
				button.enabled = false;
				button.enabled = true;
			}*/
			
			
			//Autonamer 
			
			var groupAutonamerOne = tabAutonamer.add("group");
			groupAutonamerOne.spacing = 5;
			var groupAutonamerTwo = tabAutonamer.add("group");
			
			var ruleAutonamer = groupAutonamerTwo.add("panel");
			ruleAutonamer.preferredSize = [235,0];
			
			
			var groupAutonamerThree = tabAutonamer.add("group");
			groupAutonamerThree.spacing = 5;
			
			var buttonAutonamer7 = groupAutonamerThree.add("button", undefined, "Dated Folder");
			buttonAutonamer7.preferredSize = [90, 25];
			buttonAutonamer7.helpTip = "Adds a new dated folder to your main comps folder if it doesn't already have one.";
			buttonAutonamer7.onClick = function() {
				datedFolder();
			}
			
			var buttonAutonamer0 = groupAutonamerThree.add("button", undefined, "Dupe");
			buttonAutonamer0.preferredSize = [45, 25];
			buttonAutonamer0.helpTip = 'Duplicates the selected comps keeping their names the same.';
			buttonAutonamer0.onClick = function() {
				dupe();
			}
			var buttonAutonamer5 = groupAutonamerThree.add("button", undefined, "Trim");
			buttonAutonamer5.preferredSize = [45, 25];
			buttonAutonamer5.helpTip = "Trims extra words from the end of the names of the selected comps.";
			buttonAutonamer5.onClick = function() {
				trim();
			}		
			
			/*var dividerAutonamer = groupAutonamerThree.add("panel");
			dividerAutonamer.spacing = 25;
			dividerAutonamer.preferredSize = [0,25];*/
			
			//var dividerAutonamer = groupAutonamerThree.add("statictext", undefined, "|");

			var buttonAutonamer1 = groupAutonamerThree.add("button", undefined, "Reset");
			buttonAutonamer1.preferredSize = [45, 25];
			buttonAutonamer1.helpTip = 'Resets the date and version to "_v01_01_YYMMDD_" in the names of the selected comps.';
			
			
			var buttonAutonamer2 = groupAutonamerOne.add("button", undefined, "Client");
			buttonAutonamer2.preferredSize = [45, 25];
			buttonAutonamer2.helpTip = "Versions up the client number in the names of the selected comps.";
			buttonAutonamer2.onClick = function() {
				autoClientVersionUp();
			}
			var buttonAutonamer3 = groupAutonamerOne.add("button", undefined, "Version");
			buttonAutonamer3.preferredSize = [45, 25];
			buttonAutonamer3.helpTip = "Versions up the internal number in the names of the selected comps.";
			buttonAutonamer3.onClick = function() {
				autoVersionUp();
			}
			var buttonAutonamer4 = groupAutonamerOne.add("button", undefined, "Date");
			buttonAutonamer4.preferredSize = [45, 25];
			buttonAutonamer4.helpTip = "Changes the names of the selected comps to include today's date.";
			buttonAutonamer4.onClick = function() {
				autodate();
			}
			var buttonAutonamer6 = groupAutonamerOne.add("button", undefined, "Initials");
			buttonAutonamer6.preferredSize = [45, 25];
			buttonAutonamer6.helpTip = "Changes the initials at the end of the comp to the intials in the box to the right.";
			var initialsAutonamer = groupAutonamerOne.add("edittext", undefined, "TB");
			initialsAutonamer.preferredSize = [45, 25];
			initialsAutonamer.maxLength = 2;
			initialsAutonamer.alignChildren = ["center", "center"];
			buttonAutonamer6.onClick = function() {
				var init = String(initialsAutonamer.text);
				autoInitials(init);
			}
			buttonAutonamer1.onClick = function() {			
				var init = String(initialsAutonamer.text);
				resetVersion(init);		
			}
			
			//Batch Tab
			tabBatch.orientation = "row";
			tabBatch.alignChildren = ["center", "top"];
			tabBatch.spacing = 5;
			
			var batchDropdown = tabBatch.add("dropdownlist", undefined, "Dropdown");
			batchDropdown.minimumSize = [150, 20];
			
			var itemA = batchDropdown.add("item", "Batch Renamer");
			var itemC = batchDropdown.add("item", "Batch Comp Settings");
			var itemB = batchDropdown.add("item", "Batch Font Changer");
			//var itemD = batchDropdown.add("item", "Gatherer");

			
			batchDropdown.selection = batchDropdown.items[0];

			var runButtonBatch = tabBatch.add("button", undefined, "Run");
			runButtonBatch.preferredSize = [50, 25];
			batchDropdown.helpTip = "This opens a window to batch rename all the selected comps in the project panel.";
			runButtonBatch.helpTip = "This opens a window to batch rename all the selected comps in the project panel.";
			batchDropdown.onChange = function() {
				var text = String(batchDropdown.selection.text);
				
				if(text == "Batch Renamer"){
					batchDropdown.helpTip = "This opens a window to batch rename all the selected comps in the project panel.";
					runButtonBatch.helpTip = "This opens a window to batch rename all the selected comps in the project panel.";
				}
				else if (text == "Batch Comp Settings"){
					batchDropdown.helpTip = "This opens a window to collectively change the comp settings on all the selected comps in the project panel.";
					runButtonBatch.helpTip = "This opens a window to collectively change the comp settings on all the selected comps in the project panel.";
				}
				else if (text == "Batch Font Changer"){
					batchDropdown.helpTip = "This opens a window to swap out all the fonts that are in a project.";
					runButtonBatch.helpTip = "This opens a window to swap out all the fonts that are in a project.";
				}
				
				
			}
			runButtonBatch.onClick = function() {
				var text = String(batchDropdown.selection.text);
				
				if(text == "Batch Renamer"){
					batchRenamerWindow();
				}
				else if(text == "Batch Font Changer"){
					batchFontChangerWindow();
				}
				else if(text == "Batch Comp Settings"){
					batchCompSettingsWindow();
				}
				else{
					alert("No script defined", scriptName);
				}
			}
				
			
			/*
			//Renamer
			tabRenamer.alignChildren = ["fill", "top"];
			
			var groupRenamerOne = tabRenamer.add("group");
			groupRenamerOne.margins = [9,0,1,0];

			var groupRenamerTwo = tabRenamer.add("group");
			groupRenamerTwo.margins = [9,0,1,0];

			var groupRenamerThree = tabRenamer.add("group");
			groupRenamerThree.margins = [9,0,1,0];

			var groupRenamerFour = tabRenamer.add("group");
			groupRenamerFour.margins = [9,0,1,10];

			var groupRenamerFive = tabRenamer.add("group");

			var findWhat = groupRenamerOne.add("statictext", undefined, "Find what: ");
			findWhat.minimumSize = [70, 0];
			var find = groupRenamerOne.add("edittext", undefined, "");
			find.alignment = ["fill", "top"];

			var replaceWhat = groupRenamerTwo.add("statictext", undefined, "Replace with: ");
			replaceWhat.minimumSize = [70, 0];
			var replace = groupRenamerTwo.add("edittext", undefined, "");
			replace.alignment = ["fill", "top"];

			var prefixWhat = groupRenamerThree.add("statictext", undefined, "Prefix: ");
			prefixWhat.minimumSize = [70, 0];
			
			var prefix = groupRenamerThree.add("edittext", undefined, "");
			prefix.alignment = ["fill", "top"];

			var suffixWhat = groupRenamerFour.add("statictext", undefined, "Suffix: ");
			suffixWhat.minimumSize = [70, 0];
			var suffix = groupRenamerFour.add("edittext", undefined, "");
			suffix.alignment = ["fill", "top"];

			var renameButton = groupRenamerFive.add("button", undefined, "Rename");
			renameButton.preferredSize = [70, 25];
			renameButton.helpTip = "Batch renames all selected comps at once.";
			renameButton.alignment = ["center", "top"];
			renameButton.onClick = function(){
				var f = String(find.text);
				var r = String(replace.text);
				var p = String(prefix.text);
				var s = String(suffix.text);
				renamer(f, r, p, s);
			}
			
			//Comps
			tabComps.alignChildren = ["fill", "top"];
			
			var groupCompsOne = tabComps.add("group");
			groupCompsOne.margins = [9,0,1,0];

			var groupCompsTwo = tabComps.add("group");
			groupCompsTwo.margins = [9,0,1,0];

			var groupCompsThree = tabComps.add("group");
			groupCompsThree.margins = [9,0,1,10];

			var groupCompsFour = tabComps.add("group");


			var compWidthWhat = groupCompsOne.add("statictext", undefined, "Width: ");
			compWidthWhat.minimumSize = [63, 0];
			var widthText = groupCompsOne.add("edittext", undefined, "");
			widthText.alignment = ["fill", "top"];

			var compHeightWhat = groupCompsTwo.add("statictext", undefined, "Height: ");
			compHeightWhat.minimumSize = [63, 0];
			var heightText = groupCompsTwo.add("edittext", undefined, "");
			heightText.alignment = ["fill", "top"];

			var frameRateWhat = groupCompsThree.add("statictext", undefined, "Frame Rate: ");
			frameRateWhat.minimumSize = [63, 0];
			var frameRateText = groupCompsThree.add("edittext", undefined, "");
			frameRateText.alignment = ["fill", "top"];

			var compsButton = groupCompsFour.add("button", undefined, "Change Comp Settings");
			compsButton.preferredSize = [140, 25];
			compsButton.helpTip = "Changes the comp settings for all selected comps at once.";
			compsButton.alignment = ["center", "top"];
			compsButton.onClick = function() {
				var w = String(widthText.text);
				var h = String(heightText.text);
				var fr = String(frameRateText.text);
				changeComps(w, h, fr);
			}
			*/
			
			//Expressions Tab
			tabExpression.orientation = "row";
			tabExpression.alignChildren = ["center", "top"];
			tabExpression.spacing = 5;
			
			var expressionDropdown = tabExpression.add("dropdownlist", undefined, "Dropdown");
			expressionDropdown.minimumSize = [150, 20];
			
			var item00 = expressionDropdown.add ("item", "Overshoot");
			var item01 = expressionDropdown.add ("item", "Bounce");
			var item02 = expressionDropdown.add ("item", "Drift");
			var item03 = expressionDropdown.add ("item", "Loop Both");
			var item04 = expressionDropdown.add ("item", "Link Focus Distance to Null");
			var item05 = expressionDropdown.add ("item", "Link Scale to Cam Distance");
			
			var item99 = expressionDropdown.add ("item", "Remove Expressions");

			expressionDropdown.selection = expressionDropdown.items[0];

			var runExpression = tabExpression.add("button", undefined, "Run");
			runExpression.preferredSize = [50, 25];
			
			expressionDropdown.helpTip = "Adds an expression to smoothly overshoot and then correct the motion of all selected properties, as well as effects to control the frequency/amplitude/decay of that motion.";
			runExpression.helpTip = "Adds an expression to smoothly overshoot and then correct the motion of all selected properties, as well as effects to control the frequency/amplitude/decay of that motion.";
			
			expressionDropdown.onChange = function(){
				var text = String(expressionDropdown.selection.text);
				
				if (text == "Overshoot"){
					expressionDropdown.helpTip = "Adds an expression to smoothly overshoot and then correct the motion of all selected properties, as well as effects to control the frequency/amplitude/decay of that motion.";
					runExpression.helpTip = "Adds an expression to smoothly overshoot and then correct the motion of all selected properties, as well as effects to control the frequency/amplitude/decay of that motion.";
				}
				else if (text == "Bounce"){
					expressionDropdown.helpTip = "Adds an expression to make the values of all selected properties bounce and then settle, as well as effects to control the amount and duration of that bounce.";
					runExpression.helpTip = "Adds an expression to make the values of all selected properties bounce and then settle, as well as effects to control the amount and duration of that bounce.";
				}
				else if (text == "Drift"){
					expressionDropdown.helpTip = "Adds an expression to make the value of all selected properties continue at a constant rate, as well as effects to control the speed of that rate.";
					runExpression.helpTip = "Adds an expression to make the value of all selected properties continue at a constant rate, as well as effects to control the speed of that rate.";
				}
				else if(text == "Loop Both"){
					expressionDropdown.helpTip = "Adds both a loopOut and loopIn expression to all selected properties to continue their motion in both directions, as well as effects to control how that motion is continued.";
					runExpression.helpTip = "Adds both a loopOut and loopIn expression to all selected properties to continue their motion in both directions, as well as effects to control how that motion is continued.";
				}
				else if(text == "Link Focus Distance to Null"){
					expressionDropdown.helpTip = "Creates a null and adds an expression into the focus distance of a camera to make it always focus on that null";
					runExpression.helpTip = "Creates a null and adds an expression into the focus distance of a camera to make it always focus on that null";
				}
				else if (text == "Link Scale to Cam Distance"){
					expressionDropdown.helpTip = "Adds an expression into the scale effect of a layer to make it always maintain the same relative size even as it's moved away from the camera.";
					runExpression.helpTip = "Adds an expression into the scale effect of a layer to make it always maintain the same relative size even as it's moved away from the camera.";
				}
				else if (text == "Remove Expressions"){
					expressionDropdown.helpTip = "Removes all expressions from the selected properties.";
					runExpression.helpTip = "Removes all expressions from the selected properties.";
				}
				
			}
			
			runExpression.onClick = function() {
				var text = String(expressionDropdown.selection.text);
				addExpression(text);
			}
			
			
			//About
			tabAbout.alignChildren = ["fill", "top"];
			var aboutDescription = tabAbout.add("statictext", undefined, "A collection of scripts to make life easier. Select a tab to switch between different categories of scripts. Hover your mouse over the buttons on each tab for instructions for what each script does." + "\r\r" + scriptName + " v" + version + "  |  tombloczynski.com", {multiline: true});
			tabAbout.margins = [15,10,5,0];
		 
			panel.layout.layout(true);

			return panel;
		}
			
		var panel = toolbox_buildUI(thisObj);
		panel.onResizing = panel.onResize = function() {
			//this.children[0].children[1].children[0].orientation = "column";
			this.layout.resize();


		};
		if (panel != null && panel instanceof Window){
			panel.center();
			panel.show();
		}
	}
	toolbox(this);
	
	
	//////////////////////////////////////////////////////////////////////////////////////////
	// Tidy 001 - Remove Unused Effects
	
	//This script removes all inactive effects from the project.
	function removeUnusedEffects(){
		var name = "Remove Unused Effects";
  
		app.beginUndoGroup(name);
		main(name);
		app.endUndoGroup();
	  
		// Main function.
		function main(name) {
			var length = app.project.items.length;
			var effectsRemoved = 0;
			//Searches through all items in a project panel.
			for(var i = 1; i <= length; i++){
				var item = app.project.items[i];
  
				if(item instanceof CompItem){
					var numLayers = item.layers.length;
  
					//Searches through all layers in a comp.
					for(var j = 1; j <= numLayers; j++){
						var effects = [];
						var layer = item.layers[j];
						var numProperties = 0;
						if(layer.property("Effects") != null){
							numProperties = layer.property("Effects").numProperties;
						}
						//Searches through all effects on a layer and deletes the inactive ones.
						for(var k = numProperties; k > 0; k--){
							var property;
							if(layer.property("Effects") != null){
								property = layer.property("Effects")(k);
							}
  
							if(!property.enabled){
								property.remove();
								effectsRemoved++;
							}
						}
  
					}
				}
			}	
			  
			//Conjugate sentence.
			var s = "s were";
			if(effectsRemoved == 1){
				s = " was";
			}
			alert(effectsRemoved + " inactive effect" + s + " removed.", name);
		  }
	 }
		
	
	//////////////////////////////////////////////////////////////////////////////////////////
	// Tidy 002 - Remove Empty Folders
	
	//This script removes all empty folders from the project.
	function removeEmptyFolders(){
		var name = "Remove Empty Folders";
		var foldersRemoved = 0;
		var change = true;

		app.beginUndoGroup(name);
		main(name);
		app.endUndoGroup();
	
		// Main function.
		function main(name) {
			while(change){
				removeFolders();
			}
			
			//Conjugate sentence.
			var s = "s were";
			if(foldersRemoved == 1){
				s = " was";
			}
			alert(foldersRemoved + " empty folder" + s + " removed.", name);
		}

		//Searches through all items in a project panel and deletes them.
		function removeFolders(){
			var items = app.project.items;
			var length = items.length;

			for(var i = 1; i <= length; i++){
				var item = items[i];
				if(item instanceof FolderItem){
						if(item.numItems <= 0){
							item.remove();
							foldersRemoved++;
							change = true;
							return;
						}
				}
			}	
			change = false;
		}
	}

	
	//////////////////////////////////////////////////////////////////////////////////////////
	// Tidy 003 - Organize Assets
	
	//This function organizes all assets in the Project Panel by file extention.
	function organizeAssets(){
		var name = "Organize Assets";
  
		app.beginUndoGroup(name);
		main();
		app.endUndoGroup();
	  
		//Main function.
		function main() {
			var length = app.project.items.length;
  
			var extensions = [];
			var sortables = [];
			var solids = [];
			var parents = [];
  
			//Searches through all items in a project panel and adds them and their file extensions to arrays.
			for(var i = 1; i <= length; i++){
				var item = app.project.items[i];
				if(!(item instanceof CompItem) && !(item instanceof FolderItem)){
					var type = "";
					if (item.typeName === "Footage" && item.mainSource instanceof FileSource){
						type = getFileExtension(item.file).toUpperCase();
						if(!hasElement(extensions, type) && type != ""){
							extensions.push(type);
						}
						sortables.push(item);
					}
					else if (item.mainSource instanceof SolidSource){
						type = "Solids";
						solids.push(item);
					}
					if(!hasElement(parents, item.parentFolder)){
						parents.push(item.parentFolder);
					}
					var name = item.name;
				}
			}	
			putInFolders(extensions, sortables, solids);
			removeEmptyFolders(parents);
		  }
  
		//Checks to see if an array has a particular element.
		function hasElement(array, element){
			  var length = array.length;
			  for(var i = 0; i < length; i++){
				  if(array[i] === element){
					  return true;
				  }
			  }
			  return false;
		  }
  
		//Receives an array of extensions and an array of items and puts all the items in their own folders based on extensions.
		function putInFolders(extensions, sortables, solids){
			var rootItems = app.project.rootFolder;
			var folder = checkForFolder("Asset", rootItems);
			for(var x = 0; x < extensions.length; x++){
				var extension = extensions[x];
				typeFolder = checkForFolder(extension, folder);
				var length = sortables.length;
				for(var y = 0; y < length; y++){
					var item = sortables[y];
					if (getFileExtension(item.file).toUpperCase() === extension){
						sortables[y].parentFolder = typeFolder;
					}
				}
			}
  
			var solidFolder = checkForFolder("Solid", rootItems);
			for (var x = 0; x < solids.length; x++){
				solids[x].parentFolder = solidFolder;
			}
		}
  
  
		/**
		//Looks for a folder in a given directory and returns it. If it doesn't exist, the folder is made.
		function checkForFolder(name, folder){
			var hasFolder = false;
			var index = 0;
			var newFolder = folder;
			var length = folder.numItems;
			for(var j = 1; j <= length; j++){
				var item = folder.items[j];
				var itemName = item.name.toLowerCase();
				if((itemName.indexOf(name.toLowerCase()) > -1)&& item instanceof FolderItem){
					hasFolder = true;
					index = j;
					j = folder.length;
				}
			}
			if(hasFolder){
				newFolder = folder.items[index];
				  
			}
			else{
				newFolder = folder.items.addFolder(name + "s");
			}
			return newFolder;
		}
  */
		//Returns file extension.
		function getFileExtension(file){
			var i = file.name.lastIndexOf(".");
			var fileExtention = "";
			if(i >= 0){
				fileExtention = file.name.substring(i+1);
			}
			  
			//Converts "jpeg" to "jpg"
			if (fileExtention.toLowerCase() == "jpeg"){
				fileExtention = "jpg";
			}
			return fileExtention;
		}
  
		//Searches through all items and deletes them if they are empty folders.
		function removeEmptyFolders(parents){
			var change = true;
			var newParents = [];
  
			for(var x = 0; x < parents.length; x++){
				newParents.push(parents[x]);
			}
  
  
			while(change){
				change = false;
				var length = newParents.length;
				for(var i = 0; i < length; i++){  
					var item = newParents[i];
					if(item instanceof FolderItem){
						if(item.numItems <= 0){
							if(item.parentFolder != app.project.rootFolder){
								if(!hasElement(newParents, item.parentFolder)){
									newParents.push(item.parentFolder);
								}	  
							}
							newParents[i].remove();
							newParents.splice(i,1);
							change = true;
							i = length;
						}
					}
				}	
			}	  
		}
	}
		
	
	//////////////////////////////////////////////////////////////////////////////////////////
	// Tidy 004 - Gray Everything
	
	//Searches through all items in the project panel and all layers in all comps and removes all their label colors.
	function gray(){
		var name = "Gray Everything";

	  	app.beginUndoGroup(name);
	  	main();
	  	app.endUndoGroup();
	
	   	// Main function.
	  	function main() {
		  	var length = app.project.items.length;

			//Searches through all items in a project panel.
			for(i = 1; i <= length; i++){
				var item = app.project.items[i];
				item.label = 0;

				if(item instanceof CompItem){
					var numLayers = item.layers.length;

					//Searches through all layers in a comp and deletes their color.
					for(j = 1; j <= numLayers; j++){
						var layer = item.layers[j];
						layer.label = 0;
					}
				}
			}	
		}
   	}		
		
		
	//////////////////////////////////////////////////////////////////////////////////////////
	// Tidy 005 - Autoname Layers
	
	//Searches through all adjustment layers and solids in comps throughout the project, and if they're unnamed, assigns a name to them based on their effects
	function autonameLayers(){
		var name = "Autoname Layers";

	  	app.beginUndoGroup(name);
	  	main(name);
	  	app.endUndoGroup();
	
	   	// Main function.
	  	function main(name) {
		  	var length = app.project.items.length;
			var layersRenamed = 0;

			//Searches through all items in a project panel.
			for(i = 1; i <= length; i++){
				var item = app.project.items[i];

				if(item instanceof CompItem){
					var numLayers = item.layers.length;

					//Searches through all layers in a comp.
					for(j = 1; j <= numLayers; j++){
						var layer = item.layers[j];

						//If the layer does not have a set name...
						if(!layer.isNameSet){
							//If the layer is an adjustment layer, null, or solid...
							if(layer.adjustmentLayer || (layer.source.mainSource instanceof SolidSource && !layer.nullLayer)){
								
								//If the layer has effects
								if(layer.property("Effects") != null){
									
									var isSolid = false;
									if(layer.source.mainSource instanceof SolidSource && !layer.nullLayer && !layer.adjustmentLayer){
										isSolid = true;
									}
									//Loop through all the effect names and add them to an array								
									var effectNames = [];
									var newLayerName = "";
									
									var numProperties = layer.property("Effects").numProperties;
									for (var x = 1; x <= numProperties; x++){
										if(layer.property("Effects")(x).enabled){
											effectNames.push(layer.property("Effects")(x).name.toLowerCase());
										}
									}
									//If it has no effects, name it "BREAKER"
									if (effectNames.length <= 0){
										
										if(isSolid){
											newLayerName = "";
											layersRenamed--;
										}
										else{
											newLayerName = "--BREAKER--";
										}
									}
									
									//If it has one effect, name it that effect name
									if (effectNames.length == 1){
										newLayerName += effectCheck(effectNames[0]);
									}
									
									//If it has more than one effect, name it all their names
									else if (effectNames.length > 1){
										if(isSolid){
											newLayerName = "";
										}
										newLayerName += effectCheck(effectNames[0]);
										for(var x = 1; x < effectNames.length; x++){
											newLayerName += " + " + effectCheck(effectNames[x]);
										}
										
									}
									layer.name = newLayerName;
									layersRenamed++;
															
								}
								else{
									layer.name = "BLANK";
									layerRenamed++;
								}
							}
							else if(layer.nullLayer){
								//layer.name = "NULL";
								//layersRenamed++;
							}
							/*else if(layer.source.mainSource instanceof SolidSource && !layer.nullLayer){
								
								//If the layer has effects
								if(layer.property("Effects") != null){
									
									//Loop through all the effect names and add them to an array								
									var effectNames = [];

									var numProperties = layer.property("Effects").numProperties;
									for (var x = 1; x <= numProperties; x++){
										if(layer.property("Effects")(x).enabled){
											effectNames.push(layer.property("Effects")(x).name.toLowerCase());
										}
									}

									//If it has one effect, name it that effect name
									if (effectNames.length == 1){
										newLayerName = effectCheck(effectNames[0]);
									}
									
									//If it has more than one effect, name it all their names
									else if (effectNames.length > 1){
										newLayerName = effectNames[0];
										for(var x = 1; x < effectNames.length; x++){
											newLayerName += "_" + effectNames[x];
										}
										
									}
									
									layer.name = newLayerName;
									layersRenamed++;
														
								
								//Old code to try different things for Solids. Too much work for now
								/*
								//If the layer has effects
								if(layer.property("Effects") != null){
									
									//Loop through all the active effects and add their names to an array
									var effectNames = [];

									var numProperties = layer.property("Effects").numProperties;
									for (var x = 1; x <= numProperties; x++){
										if(layer.property("Effects")(x).enabled){
											alert("going into the solid");
											effectNames.push(layer.property("Effects")(x).name.toLowerCase());
											alert("came out of solid");
										}
									}

									newLayerName = solidEffectCheck(effectNames);
									alert("did the new name");
									layer.name = newLayerName;
									layersRenamed++;
								/////////
								}
								else{
									layer.name = "BLANK";
									layerRenamed++;
								}
								
							}*/
						}
						
					}
					
					//Parent Namer (this doesn't work very well yet) (also, how would it name something if more than one thing was parented to it?)
					/**parentNamerLoop();
					parentNamerLoop();
					function parentNamerLoop(){
						for(j = 1; j <= numLayers; j++){
							var layer = item.layers[j];
							
							parentNamer(layer);
							
							//Names Parent Layers if they're nulls
							function parentNamer(layer){	
								if(layer.parent != null){
									if(layer.parent.nullLayer){
										if (layer.parent.name == "NULL"){
											layer.parent.name += ": " + layer.name;
										}
										else if (layer.parent.name == "NULL: NULL"){
											layer.parent.name = "NULL: " + layer.name;
										}
									}
								}
									
							}							
						}
					}*/
						
										
				}	
		
			}
			
			var conjugation = "s were";
			if (layersRenamed == 1){
				conjugation = " was";
			}
			alert(layersRenamed + " layer" + conjugation + " renamed.", name);

		}
		
		//Normalizes the name of one effect
		function effectCheck(effectName){
			var newLayerName = "";
			
			
			
			//Search Terms
			var grain = effectName.search("grain");
			var noise = effectName.search("noise");
			var remove = effectName.search("remove");
			var fractal = effectName.search("fractal");
			var blurr = effectName.search("blur");
			var glow = effectName.search("glow");
			var lumetri = effectName.search("lumetri");
			var key = effectName.search("key");
			var choke = effectName.search("choker");
			var wipe = effectName.search("wipe");
			var dof = effectName.search("depth of field");
			var ramp = effectName.search("gradient");
			
			var particles = effectName.search("partic");
			var flare = effectName.search("flare");
			
			if (grain > -1 && remove == -1){
				newLayerName = "GRAIN";	
			}
			else if (lumetri > -1){
				newLayerName = "LUMETRI";	
			}
			else if (noise > -1 && remove == -1 && fractal == -1){
				newLayerName = "NOISE";	
			}
			else if (fractal > -1){
				newLayerName = "FRACTAL";	
			}
			else if(blurr >- 1){
				newLayerName = "BLUR";
			}
			else if(glow >-1){
				newLayerName = "GLOW";
			}
			else if(dof >-1){
				newLayerName = "DOF";
			}
			else if(key >-1){
				newLayerName = "KEY";
			}
			else if(choke >-1){
				newLayerName = "CHOKE";
			}
			else if(wipe >-1){
				newLayerName = "WIPE";
			}
			else if(ramp >-1){
				newLayerName = "RAMP";
			}
			else if(particles >-1){
				newLayerName = "PARTICLES";
			}
			else if(flare >-1){
				newLayerName = "FLARE";
			}
			else{
				newLayerName = effectName.toUpperCase();
				newLayerName = newLayerName.replace("VR ", "");
				newLayerName = newLayerName.replace("CC ", "");
				newLayerName = newLayerName.replace("S_", "");
				newLayerName = newLayerName.replace("UNI.", "");
				//newLayerName = newLayerName.replace(/ /g, "_");
			}
			
			//If it ends in a number, get rid of it
			var lastIndex = newLayerName.lastIndexOf(" ");
			if (lastIndex > 0){
				if(!isNaN(newLayerName.substring(lastIndex, newLayerName.length))){
					newLayerName = newLayerName.substring(0, lastIndex);
				}
			}	
			
			return newLayerName;
		}
	
	
		//Finds the top most instance of a common effect that covers others up and names the solid that effect's name. Never got this working how I wanted
		function solidEffectCheck(effectNames){
			var newLayerName = "SOLID";
			alert("in solid effect check");
			var numProperties = effectNames.length;
			
			//Search Terms
			var hasParticles = -1;
			var hasFlare = -1;
			var hasRamp = -1;
			var hasFractal = -1;
			var hasWipe = -1;
			var hasStroke = -1;
			var hasSupercomp = -1;
			var hasElement = -1;
			var hasSaber = -1;
			var hasMir = -1;
			var hasForm = -1;
			var hasTao = -1;
			
			//Find which effect is on top
			for (var x = 0; x <= numProperties; x++){
				if(effectNames[x].search("partic") > -1){
					hasParticles = x;
				}
				if(effectNames[x].search("flare") > -1){
					hasFlare = x;
				}
				if(effectNames[x].search("ramp") > -1){
					hasRamp = x;
				}
				if(effectNames[x].search("fractal") > -1){
					hasFractal = x;
				}
				if(effectNames[x].search("wipe") > -1){
					hasWipe = x;
				}
				if(effectNames[x].search("stroke") > -1){
					hasStroke = x;
				}
				if(effectNames[x].search("supercomp") > -1){
					hasSupercomp = x;
				}
				if(effectNames[x].search("element") > -1){
					hasElement = x;
				}
				if(effectNames[x].search("saber") > -1){
					hasSaber = x;
				}
				if(effectNames[x].search("mir") > -1){
					hasMir = x;
				}
				if(effectNames[x].search("form") > -1){
					hasForm = x;
				}
				if(effectNames[x].search("tao") > -1){
					hasTao = x;
				}
			}
			
			var biggest = Math.max(hasParticles, hasFlare, hasRamp, hasFractal, hasWipe, hasStroke, hasSupercomp, hasElement, hasSaber, hasMir, hasForm, hasTao);
			if (biggest == hasParticles){
				newLayerName == "PARTICLES";
			}
			else if (biggest == hasFlare){
				newLayerName == "FLARE";
			}
			else if (biggest == hasRamp){
				newLayerName == "RAMP";
			}
			else if (biggest == hasFractal){
				newLayerName == "FRACTAL";
			}
			else if (biggest == hasWipe){
				newLayerName == "WIPE";
			}
			else if (biggest == hasStroke){
				newLayerName == "STROKE";
			}
			else if (biggest == hasSupercomp){
				newLayerName == "SUPERCOMP";
			}
			else if (biggest == hasElement){
				newLayerName == "ELEMENT";
			}
			else if (biggest == hasSaber){
				newLayerName == "SABER";
			}
			else if (biggest == hasMir){
				newLayerName == "MIR";
			}
			else if (biggest == hasForm){
				newLayerName == "FORM";
			}
			else if (biggest == hasTao){
				newLayerName == "TAO";
			}
			alert("did all this nonsense");
			return newLayerName;
		}
	}
	

	//////////////////////////////////////////////////////////////////////////////////////////
	// Tidy 006 - Remove Guide Layers
	
	//Searches through all the comps in the project and removes all the guide layers.
	function removeGuideLayers(){
		var name = "Remove Guide Layers";

	  	app.beginUndoGroup(name);
	  	main(name);
	  	app.endUndoGroup();
	
	   	// Main function.
	  	function main(name) {
		  	var length = app.project.items.length;
			var guideLayersRemoved = 0;
			
			//Searches through all items in a project panel.
			for(var i = 1; i <= length; i++){
				var item = app.project.items[i];

				if(item instanceof CompItem){
					var numLayers = item.layers.length;

					//Searches through all layers in a comp and deletes them if they're a guide layer.
					for(var j = numLayers; j > 0; j--){
						var layer = item.layers[j];
						if(layer.guideLayer){
							layer.remove();
							guideLayersRemoved++;
						}
					}
				}
			}
			
			var conjugation = "s were";
			if (guideLayersRemoved == 1){
					conjugation = " was";
			}
			alert(guideLayersRemoved + " guide layer" + conjugation + " removed.", name);
		}	
	}
	
	//////////////////////////////////////////////////////////////////////////////////////////
	// Tidy 007 - Hides Viewers Guides
	
	//Searches through all the comps in the project and turns off the guides in the viewport. This doesn't work. Can't find the options for activeViewer.views[0].options
	function turnOffViewerGuides(){
		var name = "Hide Viewer Guides";
		var youSure = confirm("This will open all comps in the project one by one, hide their viewer guides, then close them. Are you sure you want to do that?", name);
		
		if(youSure){
			app.beginUndoGroup(name);
			main();
			app.endUndoGroup();
		}
		
	   	// Main function.
	  	function main() {
			var count = 0;
		  	var length = app.project.items.length;
			var ogActiveViewer = app.activeViewer;

			//Searches through all items in a project panel.
			for(i = 1; i <= length; i++){
				var item = app.project.items[i];


				if(item instanceof CompItem){
					item.openInViewer();
					var activeViewer = app.activeViewer;
					activeViewer.views[0].options.guidesVisibility = false;
					activeViewer.views[0].options.rulers = false;
					app.executeCommand(app.findMenuCommandId("Close"));
					count++;
					//app.executeCommand(app.findMenuCommandId("Show Grid")); You can do commands by directly accessing them from the menu like you're a user clicking on it
					//activeViewer.views[0].options.grid = false;
					//activeViewer.views[0].options.titlesafe = false;
					

				}
			}
			
			var conjugation = "";
			if(count == 1){
				conjugation = "s";
			}
			
			alert(count + " comp" + conjugation + " were opened and their guides were hidden.", name);
			ogActiveViewer.setActive();
		}	
	}
   		
	//////////////////////////////////////////////////////////////////////////////////////////
	// Tidy 008 - Nudge Keyframes
	
	//Searches through selected comps and nudges the keyframes to the nearest whole key frame.
	function nudgeKeyframes(){
		var name = "Nudge Keyframes";

	  	app.beginUndoGroup(name);
	  	main();
	  	app.endUndoGroup();
	
	   	// Main function.
	  	function main() {
		  	var length = app.project.items.length;

			if (app.project.selection.length <= 0) {
				alert("No item selected in the project panel.", name);
				return;
			}
			app.beginUndoGroup(name);
		
			var count = 0;
			var items = app.project.selection;
			for(i = 0; i < items.length; i++){
				if(items[i] instanceof CompItem){
					count++;
					var fr = items[i].frameRate;
					var compLayers = items[i].layers;
					for(j = 1; j <= compLayers.length; j++){

						var myLayer = compLayers[j];
						var propsWithKeyframes = [];
						processProperty(myLayer);
						
						function processProperty(theProp){
							if (theProp.propertyType == PropertyType.PROPERTY){
								if(theProp.numKeys > 0){
									propsWithKeyframes.push(theProp);
								}
							}else{ // must be a group
								for (var i = 1; i <= theProp.numProperties; i++){
									processProperty(theProp.property(i));
								}
							}
						}

						for(k = 0; k < propsWithKeyframes.length; k++){
							var prop = propsWithKeyframes[k];
							if (prop.numKeys > 0){
								var keysTime = [];
								var keysValue = [];
								var keysInInterpolateType = [];
								var keysOutInterpolateType = [];								
								
								var keysInTemporalEase = [];
								var keysOutTemporalEase = [];
								var keysTemporalAutoBezier = [];
								var keysTemporalContinuous = [];
								
								var keysInSpatialTangents = [];
								var keysOutSpatialTangents = [];
								
								var keysSpatialAutoBezier = [];
								var keysSpatialContinuous = [];
								var keysRoving = [];
								
								var numKeys = prop.numKeys;
								//Gets the properties of all keys
								for(l = 1; l <= numKeys; l++){

									keysTime.push(roundFrame(items[i], prop.keyTime(l)));
																										
									if (prop.propertyValueType != PropertyValueType.CUSTOM_VALUE){
										keysValue.push(prop.keyValue(l));
									}
									else{
										keysValue.push(0);
									}

									keysInInterpolateType.push(prop.keyInInterpolationType(l));
									keysOutInterpolateType.push(prop.keyOutInterpolationType(l));
									
									
									keysInTemporalEase.push(prop.keyInTemporalEase(l));
									keysOutTemporalEase.push(prop.keyOutTemporalEase(l));

									keysTemporalAutoBezier.push(prop.keyTemporalAutoBezier(l));
									keysTemporalContinuous.push(prop.keyTemporalContinuous(l));
									
									/**if (prop.propertyValueType == PropertyValueType.TwoD_SPATIAL || prop.propertyValueType == PropertyValueType.ThreeD_SPATIAL){

										keysInSpatialTangents.push(prop.keyInSpatialTangent(l));
										keysOutSpatialTangents.push(prop.keyOutSpatialTangent(l));
										
										keysSpatialAutoBezier.push(prop.keySpatialAutoBezier(l));
										keysSpatialContinuous.push(prop.keySpatialContinuous(l));
										
										keysRoving.push(prop.keyRoving(l));

									}
									else {
										keysInSpatialTangents.push(0);
										keysOutSpatialTangents.push(0);
										
										keysSpatialAutoBezier.push(0);
										keysSpatialContinuous.push(0);
										
										keysRoving.push(0);
									}*/

								}
								
								//Removes all keyframes
								for(l = numKeys; l > 0; l--){
									if (prop.propertyValueType != PropertyValueType.CUSTOM_VALUE){
										prop.removeKey(l);
									}	
									
								}								

								//Adds new keyframes to the correct frame rate
								for(l = 1; l <= numKeys; l++){
									if (prop.propertyValueType != PropertyValueType.CUSTOM_VALUE){
										prop.setValueAtTime(keysTime[l-1], keysValue[l-1]);

									}	
									prop.setInterpolationTypeAtKey(l, keysInInterpolateType[l-1], keysOutInterpolateType[l-1]);
									//prop.setSpatialTangentsAtKey(l, keysInSpatialTangents[l-1],keysOutSpatialTangents[l-1]);
									prop.setTemporalEaseAtKey(l, keysInTemporalEase[l-1], keysOutTemporalEase[l-1]);
									prop.setTemporalAutoBezierAtKey(l, keysTemporalAutoBezier[l-1]);
									prop.setTemporalContinuousAtKey(l, keysTemporalContinuous[l-1]);
									
									/**if (prop.propertyValueType == PropertyValueType.TwoD_SPATIAL || prop.propertyValueType == PropertyValueType.ThreeD_SPATIAL){

										prop.setSpatialTangents.push(l, keysInSpatialTangent[l-1], keysOutSpatialTangent[l-1]);
										
										prop.setSpatialAutoBezierAtKey(l, keysSpatialAutoBezier[l-1]);
										prop.setSpatialContinuousAtKey(l, keySpatialContinuous[l-1]);
										
										prop.setRovingAtKey(l, keysRoving[l-1]);

									}*/
									
								}
		
								
							}
						}
					}
				}
			}
			var conj = " comps had all their";
			if (count == 1){
				conj = " comp had all its";
			}
			alert(count + conj + " keyframes nudged.", name);
			app.endUndoGroup();
		}
		
		
			
		
		
		function roundFrame(comp, time){
			return Math.round(comp.frameRate*time)/comp.frameRate;
		}
   	}		

	//////////////////////////////////////////////////////////////////////////////////////////
	// Tidy 009 - Organize Precomps
	
	//This function organizes all comps in the Project Panel if it thinks that they're precomps.
	function gatherPrecomps(){
		
		main();
		
		function main(){
			var length = app.project.items.length;

			var extensions = [];
			var sortables = [];
			var parents = [];
			var parentComps = [];

			//Searches through all items in a project panel and if they have a parent comp, adds them to an array.
			for(var i = 1; i <= length; i++){
				var item = app.project.items[i];
				if(item instanceof CompItem){
					var type = "";
					if(!hasElement(parents, item.parentFolder)){
						parents.push(item.parentFolder);
					}
					if(item.usedIn.length == 1){
						sortables.push(item);
						
					}
					else if(item.usedIn.length > 1){
						sortables.push(item);
						
					}
					var name = item.name;
				}
			}	
		}
		
		//Receives an array of extensions and an array of items and puts all the items in their own folders based on extensions.
		function putInFolders(extensions, sortables){
			var rootItems = app.project.rootFolder;
			var folder = checkForFolder("Pre", rootItems);
			for(var x = 0; x < extensions.length; x++){
				var extension = extensions[x];
				typeFolder = checkForFolder(extension, folder);
				var length = sortables.length;
				for(var y = 0; y < length; y++){
					sortables[y].parentFolder = typeFolder;
				}
				
			}
		}
		
		//Looks for a folder in a given directory and returns it. If it doesn't exist, the folder is made.
		function checkForFolder(name, folder){
			var hasFolder = false;
			var index = 0;
			var newFolder = folder;
			var length = folder.numItems;
			for(var j = 1; j <= length; j++){
				var item = folder.items[j];
				var itemName = item.name.toLowerCase();
				if((itemName.indexOf(name.toLowerCase()) > -1)&& item instanceof FolderItem){
					hasFolder = true;
					index = j;
					j = folder.length;
				}
			}
			if(hasFolder){
				newFolder = folder.items[index];
				  
			}
			else{
				newFolder = folder.items.addFolder(name + "s");
			}
			return newFolder;
		}
		
		//Checks to see if an array has a particular element.
		function hasElement(array, element){
			  var length = array.length;
			  for(var i = 0; i < length; i++){
				  if(array[i] === element){
					  return true;
				  }
			  }
			  return false;
		}
		
	}
	
	
	//////////////////////////////////////////////////////////////////////////////////////////
	// Tidy 010 - Replace Fonts
	
	//Finds the fonts that are in the active comp, and replaces other fonts in the project with them
	function replaceFonts(oldFont, font){
		var name = "Batch Font Changer";
		var oldFont = String(oldFont);
		var font = String(font);
		
		if (oldFont == font){
			alert("Same font selected for both. Please select two different fonts in order to change them.", name);
		}
		else{
			app.beginUndoGroup(name);
		
			var fontsChanged = 0;
			
			var length = app.project.items.length;
			//Searches through all items in a project panel.
			for(var i = 1; i <= length; i++){
				var item = app.project.items[i];

				if(item instanceof CompItem){
					var numLayers = item.layers.length;

					//Searches through all layers in a comp and switches their fonts
					for(var j = 1; j <= numLayers; j++){
						var layer = item.layers[j];
						if (layer instanceof TextLayer){
							if (layer.property("Source Text").value.font.match(oldFont) != null){
								
								var textProp = layer.property("Source Text");
								if(textProp.numKeys > 0){
									var textDocument = new TextDocument("Test");
									for(var x = 1; x <= textProp.numKeys; x++){
										
										textDocument = textProp.valueAtTime(textProp.keyTime(x), true);

										textDocument.font = font;
										
										textProp.setValueAtTime(textProp.keyTime(x), textDocument);

									}
								}
								else{
									var textDocument = new TextDocument("Test");
									textDocument = textProp.value;

									textDocument.font = font;

								}
								
								fontsChanged++;
								
							}
							
						}
					}
				}
			}
					
				
			
			
			var conjugation = "s had their fonts";
			if (fontsChanged == 1){
				conjugation = " had its font";
			}
			alert(fontsChanged + " text layer" + conjugation + " changed.", name);
			app.endUndoGroup();
				
		}
			
	}
	
	
	//////////////////////////////////////////////////////////////////////////////////////////
	// Autonamer
	//This collect of scripts automatically versions up and dates the naming scheme of The Refinery Creative's comps.


	//Creates a folder with today's date in the main Comps folder
	function datedFolder(){
		app.beginUndoGroup("Make Dated Folder");
		var date = currentDate();
		var rootFolder = app.project.rootFolder;
		var compFolder = checkForFolder("Comp", rootFolder);
		var folder = checkForFolder(date, compFolder);
		app.endUndoGroup;
	}

	//Looks for a folder in a given directory and returns it. If it doesn't exist, the folder is made.
	function checkForFolder(name, folder){
		var hasFolder = false;
		var index = 0;
		var newFolder = folder;
		var length = folder.numItems;
		for(var j = 1; j <= length; j++){
			var item = folder.items[j];
			var itemName = item.name.toLowerCase();
			if((itemName.indexOf(name.toLowerCase()) > -1)&& item instanceof FolderItem){
				hasFolder = true;
				index = j;
				j = folder.length;
			}
		}
		if(hasFolder){
			newFolder = folder.items[index];
			  
		}
		else{
			if (!isNaN(parseInt(name))){
				newFolder = folder.items.addFolder(name);
			}
			else{
				newFolder = folder.items.addFolder(name + "s");
			}
		}
		return newFolder;
	}
  

	function autodate() {
		if (app.project.selection.length <= 0) {
			alert("No item selected in the project panel.", "Autodate");
			return;
		}
		app.beginUndoGroup("Autodate");
		
		var items = app.project.selection;
		for(i = 0; i < items.length; i++){
			if(items[i] instanceof CompItem){
				items[i].name = replaceDate(items[i].name);
			}
			if (parseFloat(app.version) < 9) items[i].name = items[i].name.substring(0,29); // CS3 has a 31 character limit
		}
		for(i = 0; i < items.length; i++){
			app.project.selection[i].name = items[i].name;
		}
		app.endUndoGroup();
	}
	
	function dupe() {
		if (app.project.selection.length <= 0) {
			alert("No item selected in the project panel.", "Dupe");
			return;
		}
		app.beginUndoGroup("Dupe");
		var items = app.project.selection;
		for(i = 0; i < items.length; i++){
			if(items[i] instanceof CompItem){
				var item = items[i].duplicate();
				item.name = items[i].name;
			}
		}
		app.endUndoGroup();
	}
	
	function trim(){
		if (app.project.selection.length <= 0) {
			alert("No item selected in the project panel.", "Reset Version");
			return;
		}
		app.beginUndoGroup("Trim");
		var items = app.project.selection;
		for(i = 0; i < items.length; i++){
			if(items[i] instanceof CompItem){
				items[i].name = trimLastWord(items[i].name);
			}
			if (parseFloat(app.version) < 9) items[i].name = items[i].name.substring(0,29); // CS3 has a 31 character limit
		}
		for(i = 0; i < items.length; i++){
			app.project.selection[i].name = items[i].name;
		}
		app.endUndoGroup();
	}
	
	function resetVersion(intials) {
		if (app.project.selection.length <= 0) {
			alert("No item selected in the project panel.", "Reset Version");
			return;
		}
		app.beginUndoGroup("Reset Version");
		var items = app.project.selection;
		for(i = 0; i < items.length; i++){
			if(items[i] instanceof CompItem){
				items[i].name = zeroOut(items[i].name);
				items[i].name = replaceDate(items[i].name);
				items[i].name = replaceInitials(items[i].name, intials);

			}
			if (parseFloat(app.version) < 9) items[i].name = items[i].name.substring(0,29); // CS3 has a 31 character limit
		}
		for(i = 0; i < items.length; i++){
			app.project.selection[i].name = items[i].name;
		}
		app.endUndoGroup();
	}
	
	function autoInitials(initials) {
		
		if (app.project.selection.length <= 0) {
			alert("No item selected in the project panel.", "Version Up");
			return;
		}
		app.beginUndoGroup("Intials");
		
		var items = app.project.selection;
		for(i = 0; i < items.length; i++){
			if(items[i] instanceof CompItem){
				items[i].name = replaceInitials(items[i].name, initials);
			}
			if (parseFloat(app.version) < 9) items[i].name = items[i].name.substring(0,29); // CS3 has a 31 character limit
		}
		for(i = 0; i < items.length; i++){
			app.project.selection[i].name = items[i].name;
		}
		app.endUndoGroup();
	}
	
	function autoVersionUp() {
		if (app.project.selection.length <= 0) {
			alert("No item selected in the project panel.", "Version Up");
			return;
		}
		app.beginUndoGroup("Version Up");
		
		var items = app.project.selection;
		for(i = 0; i < items.length; i++){
			if(items[i] instanceof CompItem){
				items[i].name = versionUp(items[i].name);
			}
			if (parseFloat(app.version) < 9) items[i].name = items[i].name.substring(0,29); // CS3 has a 31 character limit
		}
		for(i = 0; i < items.length; i++){
			app.project.selection[i].name = items[i].name;
		}
		app.endUndoGroup();
	}
	
	function autoClientVersionUp() {
		if (app.project.selection.length <= 0) {
			alert("No item selected in the project panel.", "Client Version Up");
			return;
		}
		app.beginUndoGroup("Client Version Up");
		
		var items = app.project.selection;
		for(i = 0; i < items.length; i++){
			if(items[i] instanceof CompItem){
				items[i].name = clientVersionUp(items[i].name);
			}
			if (parseFloat(app.version) < 9) items[i].name = items[i].name.substring(0,29); // CS3 has a 31 character limit
		}
		for(i = 0; i < items.length; i++){
			app.project.selection[i].name = items[i].name;
		}
		app.endUndoGroup();
	}
	
	function replaceDate(compName){
		var today = currentDate();		
		compName = compName.replace(/\d{6,}/, today); //Regex for 6 digit character
		return compName;
	}
	
	function replaceInitials(compName, initials){
		
		var oldVersion = compName.match(/\d{6}_([A-Z]|[a-z]){2,3}/); //Regex for 2 of any letter that comes after a 6 digit number.
		if (oldVersion !== null){		
			var newVersion = oldVersion[0].substring(0,7) + initials;
			compName= compName.replace(/\d{6}_([A-Z]|[a-z]){2,3}/, newVersion);
		}
		return compName;
	}
	
	function versionUp(compName){
		
		// Find the last occurrence of two digits in the string
		var lastTwoDigitsIndex = -1;
		for (var i = compName.length - 3; i >= 0; i--) {
				
			if ((compName[i-1] == "_") && !isNaN(parseInt(compName[i])) && !isNaN(parseInt(compName[i + 1])) && (compName[i+2] == "_")) {
				
				lastTwoDigitsIndex = i;
				break;
			}
		}
		// If two digits were found, increment them
		if (lastTwoDigitsIndex != -1) {
			
			var number = parseInt(compName.substr(lastTwoDigitsIndex, lastTwoDigitsIndex + 2));
			number++;
			var leadingNumber = String(0);
			if(number>9){
				leadingNumber = "";
			}
			var incrementedNumber = leadingNumber + String(number); 
			return compName.substr(0, lastTwoDigitsIndex) + incrementedNumber + compName.substr(lastTwoDigitsIndex + 2);
		} else {
			// If no two digits found, return the original string
			return compName;
		}
		
		/*var oldVersion = compName.match(/_\d{2}_/);
		
		if (oldVersion !== null){
			var oldInt = parseInt(oldVersion[0].substring(1,3));
			var leadingNumber = String(0);
		
			if(oldInt >= 9){ leadingNumber = "" };
			
			var newVersion = "_" + leadingNumber + String(oldInt + 1) + "_";
			
			compName= compName.replace(/_\d{2}_/, newVersion);
			
			
		}*/
	}
	
	function clientVersionUp(compName){
		
		var oldClientVersion = compName.match(/_[Vv]\d{2}_/);
		
		if (oldClientVersion !== null){
			var oldClientInt = parseInt(oldClientVersion[0].substring(2,4));
			var clientLeadingNumber = String(0);
		
			if(oldClientInt >= 9){ clientLeadingNumber = "" };
			
			var newClientVersion = "_v" + clientLeadingNumber + String(oldClientInt + 1) + "_";
		
			compName = compName.replace(/_[Vv]\d{2}_/, newClientVersion);
		}
		
		//compName= compName.replace(/_\d{2}_/, "_01_");
		
		// Find the last occurrence of two digits in the string
		var lastTwoDigitsIndex = -1;
		for (var i = compName.length - 3; i >= 0; i--) {
				
			if ((compName[i-1] == "_") && !isNaN(parseInt(compName[i])) && !isNaN(parseInt(compName[i + 1])) && (compName[i+2] == "_")) {
				
				lastTwoDigitsIndex = i;
				break;
			}
		}
		
		compName = compName.substr(0, lastTwoDigitsIndex) + "01" + compName.substr(lastTwoDigitsIndex + 2);
		
		return compName;
	}
	
	function trimLastWord(compName){
		
		var lastIndex = compName.lastIndexOf(" ");
		var newCompName = compName;
		if (lastIndex > 0){
			newCompName = compName.substring(0, lastIndex);
		}
		
		return newCompName;
	}
	
	function zeroOut(compName){	
		if (!compName.match(/_[Vv]\d{2}_\d{2}_\d{6,}_([A-Z]|[a-z]){2,3}/)){
			compName = compName + "_v01_01_000000_XX";
		}
		compName = compName.replace(/_[Vv]\d{2}_\d{2}_/, "_v01_01_");
		return compName;
	}
	
	
	function currentDate(){
		
		var today = new Date();
		var dd = String(today.getDate());
		if (parseInt(dd) < 10){dd = "0" + dd;}
		var mm = String(today.getMonth() + 1); //January is 0!
		if (parseInt(mm) < 10){mm = "0" + mm;}
		var yy = String(today.getFullYear()).substring(2,4);
		
		today = yy + mm + dd;
		
		return today;
	}

	//////////////////////////////////////////////////////////////////////////////////////////
	// Renamer
	//These functions batch-rename selected comps in the project panel.


	function renamer(replace, replaceWith, prefix, suffix){
		var name = "Renamer";
		 
		 var selectedItems = app.project.selection.slice(0); // Make a copy for safety
		 if (selectedItems.length <= 0) {
			alert("No item selected in the project panel.", name);
			return;
		 }
		 else if(replace == "" && replaceWith != ""){
			 alert("No text to find entered.", name);
			 return;
		 }
		 else if((replace != "" && replaceWith != "") || prefix != "" || suffix != ""){
			rename(replace, replaceWith, prefix, suffix);
		 }
		 else{
			 alert("No text entered.", name);
			 return;
		 }
	}

	 // This function accepts a string to be renamed, the replacement string, a prefix string, and a suffix string and batch renames selected items in the Project Panel accordingly.
	 function rename(replace, replaceWith, prefix, suffix) {
		app.beginUndoGroup("Renamer");
		var newItems = app.project.selection;
		var items = app.project.selection;
		for(i = 0; i < items.length; i++){
			newItems[i].name = prefix + newItems[i].name.replace(replace, replaceWith) + suffix;
			if (parseFloat(app.version) < 9) newItems[i].name = newItems[i].name.substring(0,29); // CS3 has a 31 character limit
		}
		for(i = 0; i < items.length; i++){
			items[i] = newItems[i].name;
		}
		app.endUndoGroup();
	}


	//////////////////////////////////////////////////////////////////////////////////////////
	// Change Comp Settings
	//These functions batch-change the settings of selected comps in the project panel.
	
	function changeComps(w, h, fr){
		var name = "Change Comp Settings";
		
		var selectedItems = app.project.selection.slice(0); // Make a copy for safety
				  
		if (selectedItems.length <= 0) {
			alert("No item selected in the project panel.", name);
			return;
		}
				 	 
		var doesntHaveComp = true;
		for(i = 0; i < selectedItems.length; i++){
			if(selectedItems[i] instanceof CompItem){
				doesntHaveComp = false;
			}
		}
			
		if(doesntHaveComp){
			alert("No comps selected in the project panel.", name);
			return;
		}					
				
		//Make sure someone entered text.
		if(w == "" && h == "" && fr == ""){
			alert("No text entered.", scriptName);
			return;
		}
				 
		//Make sure all items are numbers.
		if(isNaN(w)){
			alert("Width is not a number.", scriptName);
			return;
		}
		else if(isNaN(h)){
			alert("Height is not a number.", scriptName);
			return;
		}
		else if(isNaN(fr)){
			alert("Frame Rate is not a number.", scriptName);
			return;
		}	
				 
		//Check for inproper numbers.
		if(w < 0 || h < 0 || fr < 0){
			alert("Negative values are not allowed.", scriptName);
			return;
		}
		else if (w > 30000 || h > 30000){
			alert("Comp cannot be set that big.", scriptName);
			return;
		}
		else if ((w != "" && w < 1) || (h != "" && h < 1)){
			alert("Comp cannot be set that small.", scriptName);
			return;
		}
		else if(fr > 999){
			alert("Frame rate cannot be set that high.", scriptName);
			return;
		}
		else if (fr != "" && fr < 1){
			alert("Frame rate cannot be set that low.", scriptName);
			return;
		}
				 
		changeCompSettings(h, w, fr, name);

	}
	
	// This function parents all layers that aren't yet parented to a new parent given.
	function makeParentLayerOfAllUnparented(theComp, newParent){
		for (var i = 1; i <= theComp.numLayers; i++) {
			var curLayer = theComp.layer(i);
			var wasLocked = curLayer.locked;
			curLayer.locked = false;
			if (curLayer != newParent && curLayer.parent == null) {
				curLayer.parent = newParent;
			}
			curLayer.locked = wasLocked;
		}
	}
 
	// This function accepts a new height, a new width, and a new frame rate, and, batch renames selected items in the Project Panel accordingly.
	  function changeCompSettings(h, w, fr, name) {
			app.beginUndoGroup("Batch Comp Settings");
			var numberOfComps = 0;
			
			var items = app.project.selection;
			for(i = 0; i < items.length; i++){
				if(items[i] instanceof CompItem){
					numberOfComps++;
					var null3DLayer = items[i].layers.addNull();
					null3DLayer.threeDLayer = true;
					
					var compWidth = items[i].width;
					var compHeight = items[i].height;
					
					null3DLayer.position.setValue([0,0,0]);
				
					makeParentLayerOfAllUnparented(items[i], null3DLayer);
					
					if(w != ""){
						items[i].width = parseInt(w);
					}
					if(h != ""){
						items[i].height = parseInt(h);
					}
					if(fr != ""){
						items[i].frameRate = parseFloat(fr);
					}
					
					widthFactor = compWidth/2 - items[i].width/2;
					heightFactor = compHeight/2 - items[i].height/2;
					
					null3DLayer.position.setValue([-widthFactor,-heightFactor,0]);
					null3DLayer.remove();

				}
			}
			
			for(i = 0; i < items.length; i++){
				app.project.selection[i].width = items[i].width;
			} 
			
			var conjugation = " comps have had their settings changed.";
			if(numberOfComps == 1){
				conjugation = " comp has had its settings changed.";
			}
			
			alert(numberOfComps + conjugation, name);
			app.endUndoGroup();
		}



	//////////////////////////////////////////////////////////////////////////////////////////
	// Misc 01 - Folder Renderer
	// This script adds selected comps from the Project Panel to the render queue, maintaining the same folder structure.
	
	function folderRenderer(){
		var name = "Folder Renderer";
		var separator = "\\";
		var hasComp = false;
		var hasFolder = false;
		var base_path;

		main();
  
		 // Main function.
		function main() {
		  	//Changes separator for directories to the other backslash if you're on a Mac.
		  	if ($.os.indexOf("Mac") == -1)
		  	{
				separator = "/";
		  	}
		  	var items = app.project.selection.slice(0);
		 	if (items.length <= 0) {
				alert("No item selected in the project panel.", name);
				return;
		  	}
		  	else{
				app.beginUndoGroup(name);
			  	if(items.length >1){
					items = removeDuplicates(items);
			  	}
			  	var renderPath = "";
			  	renderFolders(items, separator, renderPath, 0);
			  	if(!hasComp){
				  alert("No comps or folders containing comps were selected in the project panel.", name);
			  	}
			  	app.endUndoGroup();
		  	}		  
		}
	  
		//Checks to see if an item is a folder.
		function checkFolder(item){
			return item instanceof FolderItem;
		}
	
		//Parses through items and returns either an array of all top-level folders, or if no top-level folders, then the items themselves.
		function removeDuplicates(items){
			var folders = [];
			for (i = 0; i < items.length; i++){
				if (items[i] instanceof FolderItem){
					hasFolder = true;
					folders.push(items[i]);
				}
			}
			var comps = [];
			for (i = 0; i < items.length; i++){
				var hasParent = false;
				for (j = 0; j < folders.length; j++){
					if (items[i].parentFolder == folders[j]){
						hasParent = true;
					}
				}
				if(!hasParent){
					comps.push(items[i]);
				}
			}
			var unduplicatedFolders  = [];
			if(hasFolder){
				for (i = 0; i < folders.length; i++){
					var hasDuplicate = false;
					for(j = 0; j < folders.length; j++){
						if (folders[i].parentFolder == folders[j])
						{
							hasDuplicate = true;
						}
					}
					if(!hasDuplicate){
						unduplicatedFolders.push(folders[i]);
					}
				}
			}
	   
			var newItems = comps;
			newItems.concat(unduplicatedFolders);
			return newItems;
	}
	
	//If items are a folder, renders all of their contents to a directory in a folder of the same name.
	function renderFolders(items, separator, renderPath, i){
		var length = items.length - 1;
		if(i > 0){
			length += i;
		}
		for(i; i <= length; i++){
			  if (items[i] instanceof FolderItem) {
				  hasFolder = true;
				  var folderItems = items[i].items;
				  var newRenderPath = renderPath + separator + items[i].name;
				  renderFolders(folderItems, separator, newRenderPath, 1);
			  }
			  else if(items[i] instanceof CompItem){
				  var comp = items[i];
				  hasComp = true;
				  setRenderPath(comp, renderPath);
			  }
		  }
	}
	
	// Adds comp to render queue with new render location.
	function setRenderPath(comp, renderLocation){
	  app.project.renderQueue.items.add(comp);
	  var lastItem = app.project.renderQueue.numItems;		
	  var om1 = app.project.renderQueue.item(lastItem).outputModule(1);
	  var file_name = File.decode( om1.file.name ); // Name contains special character, space?
	  
	  if(base_path == null){
		  base_path = app.project.renderQueue.item(1).outputModule(1).getSettings(GetSettingsFormat.STRING)["Output File Info"]["Base Path"];
	  }
	  
	  om1.file = new File(base_path + renderLocation + separator + file_name);
	  
	  
	}
 }


	////////////////////////////////////////////////////////////////////////	//////////////////
	// Misc 02 - Add Expression
	// This script inserts an expression into selected properties.
	
	function addExpression(expression){
		var name = "Add Expression";
		
		var ogExpression = expression;
		expression = getExpression(expression);


		if(app.project.activeItem == null){
			alert("No comp selected.", name);
		}
		else if(expression == "Nothing"){
			alert("No expression found", name);
		}
		else{
			var comp = app.project.activeItem;
			if(comp.selectedProperties.length <= 0){
				alert("No properties are selected that an expression can be added into.", name);
			}
			else{
				if(ogExpression == "Link Focus Distance to Null" && comp.selectedProperties[0].name != "Focus Distance"){
					alert("This expression can only be added into the Focus Distance property of a Camera.", name);
				}
				else if(ogExpression == "Link Scale to Cam Distance" && comp.selectedProperties[0].name != "Scale"){
					alert("This expression can only be added into the Scale property of a layer.", name);
				}
				else{
					main();
				}
				
			}
		}
  
		 // Main function.
		function main() {
			app.beginUndoGroup(name);

			var count = 0;
		  	var items = comp.selectedProperties.slice(0);
			for(var x = 0; x < items.length; x++){
				if(items[x].canSetExpression){
					count++;
					var propertyLayer = getLayer(items[x]);

					var dimensions = 1;
					if(items[x].value instanceof Array){
						if((items[x].name == "Position" || items[x].name == "Scale" || items[x].name == "Anchor Point") && !propertyLayer.threeDLayer){
							dimensions = 2;
						}
						else if((items[x].name == "Position" || items[x].name == "Scale" || items[x].name == "Anchor Point") && propertyLayer.threeDLayer){
							dimensions = 3;
						}
						else {
							dimensions = items[x].value.length;
						}
					}
					if(ogExpression == "Loop Both"){
						loopEffects(propertyLayer);
					}
					if(ogExpression == "Bounce"){
						bounceEffects(propertyLayer);
					}
					if(ogExpression == "Loop Out"){
						loopOutEffects(propertyLayer);
					}
					if(ogExpression == "Loop In"){
						loopInEffects(propertyLayer);
					}
					if(ogExpression == "Loop Both Old"){
						loopBothEffects(propertyLayer);
					}
					if(ogExpression == "Drift"){
						driftEffects(propertyLayer, dimensions);
					}
					if(ogExpression == "Link Focus Distance to Null"){
						addFocusTargetNull();
					}
					if(ogExpression == "Link Scale to Cam Distance"){
						//make a function that creates a null with a certain name
					}
					if(ogExpression == "Overshoot"){
						overshootEffects(propertyLayer);
					}
					items[x].expression = expression;
				}
			}
			
			var conjugation = "ies.";
			if (count == 1){
				conjugation = "y."
			}
			if(ogExpression == "Remove"){
				alert('Expressions were removed from ' + count + ' propert' + conjugation, name);
			}
			else{
				alert('The expression "' + ogExpression + '" was added to ' + count + ' propert' + conjugation, name);
			}
			app.endUndoGroup();
		}
		
		
		function addFocusTargetNull(){
			var comp = app.project.activeItem;
									
									


			// Search through layers if there's already a layer called 
			for(var i = 1; i <= comp.numLayers; i++){
				if(comp.layers[i].nullLayer && comp.layers[i].name == "TARGET_NULL"){
					return null;
				}
			}
									

			// Create a null layer.
			var null3DLayer = comp.layers.addNull();
				
			// Find active camera if there is one and its color
			var cameraIndex = 1;
			var cameraColor = 0;
			
			for(var i = 1; i <= comp.numLayers ; i++){
				if(comp.layers[i] instanceof CameraLayer && comp.layers[i].active){
					cameraIndex = i;
					cameraColor = comp.layers[i].label;
					i = comp.numLayers;
				}
			}

			// Set up the null
			null3DLayer.threeDLayer = true;

			null3DLayer.position.setValue([comp.width/2,comp.height/2,0]);

			null3DLayer.moveAfter(comp.layers[cameraIndex]);

			null3DLayer.label = cameraColor;
			null3DLayer.name = "FOCUS_TARGET";
		}
		
		// Function that adds the effects for the drift expression.
		function driftEffects(llayer, dimensions){
			
			var effectsProperty = llayer.property("ADBE Effect Parade");
			
			if (effectsProperty.property("Drift") == null && dimensions == 1){
				var slider = effectsProperty.addProperty("ADBE Slider Control");
				var sliderIndex = slider.propertyIndex; // Store 'slider' effect index so it can be reused later
				effectsProperty.property(sliderIndex).name = "Drift";
				effectsProperty.property(sliderIndex).property("Slider").setValue(0);
			}
			if (effectsProperty.property("Drift X") == null && dimensions >= 2){
				var sliderX = effectsProperty.addProperty("ADBE Slider Control");
				var sliderXIndex = sliderX.propertyIndex; // Store 'slider' effect index so it can be reused later
				effectsProperty.property(sliderXIndex).name = "Drift X";
				effectsProperty.property(sliderXIndex).property("Slider").setValue(0);
			}
			if (effectsProperty.property("Drift Y") == null && dimensions >= 2){
				var sliderY = effectsProperty.addProperty("ADBE Slider Control");
				var sliderYIndex = sliderY.propertyIndex; // Store 'slider' effect index so it can be reused later
				effectsProperty.property(sliderYIndex).name = "Drift Y";
				effectsProperty.property(sliderYIndex).property("Slider").setValue(0);
			}
			if (effectsProperty.property("Drift Z") == null && dimensions == 3){
				var sliderZ = effectsProperty.addProperty("ADBE Slider Control");
				var sliderZIndex = sliderZ.propertyIndex; // Store 'slider' effect index so it can be reused later
				effectsProperty.property(sliderZIndex).name = "Drift Z";
				effectsProperty.property(sliderZIndex).property("Slider").setValue(0);
			}
										
		}
		
		// Function that adds the effects for the overshoot expression.
		function overshootEffects(llayer){
			
			var effectsProperty = llayer.property("ADBE Effect Parade");
			
			if (effectsProperty.property("Amp") == null){
				var slider = effectsProperty.addProperty("ADBE Slider Control");
				var sliderIndex = slider.propertyIndex; // Store 'slider' effect index so it can be reused later
				effectsProperty.property(sliderIndex).name = "Amp";
				effectsProperty.property(sliderIndex).property("Slider").setValue(.2);
			}
			if (effectsProperty.property("Freq") == null){
				var sliderY = effectsProperty.addProperty("ADBE Slider Control");
				var sliderYIndex = sliderY.propertyIndex; // Store 'slider' effect index so it can be reused later
				effectsProperty.property(sliderYIndex).name = "Freq";
				effectsProperty.property(sliderYIndex).property("Slider").setValue(2);
			}
			if (effectsProperty.property("Decay") == null){
				var sliderZ = effectsProperty.addProperty("ADBE Slider Control");
				var sliderZIndex = sliderZ.propertyIndex; // Store 'slider' effect index so it can be reused later
				effectsProperty.property(sliderZIndex).name = "Decay";
				effectsProperty.property(sliderZIndex).property("Slider").setValue(10);
			}
										
		}
		
		// Function that adds the effects for the bounce expression.
		function bounceEffects(llayer){
			
			var effectsProperty = llayer.property("ADBE Effect Parade");
			
			if (effectsProperty.property("e") == null){
				var slider = effectsProperty.addProperty("ADBE Slider Control");
				var sliderIndex = slider.propertyIndex; // Store 'slider' effect index so it can be reused later
				effectsProperty.property(sliderIndex).name = "e";
				effectsProperty.property(sliderIndex).property("Slider").setValue(.7);
			}
			if (effectsProperty.property("g") == null){
				var sliderY = effectsProperty.addProperty("ADBE Slider Control");
				var sliderYIndex = sliderY.propertyIndex; // Store 'slider' effect index so it can be reused later
				effectsProperty.property(sliderYIndex).name = "g";
				effectsProperty.property(sliderYIndex).property("Slider").setValue(5000);
			}
			if (effectsProperty.property("nMax") == null){
				var sliderZ = effectsProperty.addProperty("ADBE Slider Control");
				var sliderZIndex = sliderZ.propertyIndex; // Store 'slider' effect index so it can be reused later
				effectsProperty.property(sliderZIndex).name = "nMax";
				effectsProperty.property(sliderZIndex).property("Slider").setValue(9);
			}
										
		}
		
		// Function that adds the effects for the loop expression.
		function loopEffects(llayer){
			var effectsProperty = llayer.property("ADBE Effect Parade");
				if (effectsProperty.property("Loop") == null){
					var menu = effectsProperty.addProperty("ADBE Dropdown Control");
					var menuIndex = menu.propertyIndex
					effectsProperty.property(menuIndex).menu.setPropertyParameters(["Loop Both", "Loop In", "Loop In", "Off"]);
					effectsProperty.property(menuIndex).name = "Loop";						
				}
				if (effectsProperty.property("Loop Style") == null){
					var menu = effectsProperty.addProperty("ADBE Dropdown Control");
					var menuIndex = menu.propertyIndex
					effectsProperty.property(menuIndex).menu.setPropertyParameters(["Continue", "Cycle", "Pingpong"]);
					effectsProperty.property(menuIndex).name = "Loop Style";						
				}
		}
		
		// Function that adds the effects for the loopOut expression.
		function loopOutEffects(llayer){
			var effectsProperty = llayer.property("ADBE Effect Parade");
				if (effectsProperty.property("Loop Out") == null){
					var menu = effectsProperty.addProperty("ADBE Dropdown Control");
					var menuIndex = menu.propertyIndex
					effectsProperty.property(menuIndex).menu.setPropertyParameters(["Continue", "Cycle", "Pingpong", "Off"]);
					effectsProperty.property(menuIndex).name = "Loop Out";						
				}
		}
		
		// Function that adds the effects for the loopBoth expression.
		function loopBothEffects(llayer){
			var effectsProperty = llayer.property("ADBE Effect Parade");
				if (effectsProperty.property("Loop In") == null){
					var menu = effectsProperty.addProperty("ADBE Dropdown Control");
					var menuIndex = menu.propertyIndex
					effectsProperty.property(menuIndex).menu.setPropertyParameters(["Continue", "Cycle", "Pingpong", "Off"]);
					effectsProperty.property(menuIndex).name = "Loop Both";						
				}
				if (effectsProperty.property("Loop Out") == null){
					var menu = effectsProperty.addProperty("ADBE Dropdown Control");
					var menuIndex = menu.propertyIndex
					effectsProperty.property(menuIndex).menu.setPropertyParameters(["Continue", "Cycle", "Pingpong", "Off"]);
					effectsProperty.property(menuIndex).name = "Loop Both";						
				}
		}
		
		// Function that adds the effects for the loopIn expression.
		function loopInEffects(llayer){
			var effectsProperty = llayer.property("ADBE Effect Parade");
				if (effectsProperty.property("Loop In") == null){
					var menu = effectsProperty.addProperty("ADBE Dropdown Control");
					var menuIndex = menu.propertyIndex
					effectsProperty.property(menuIndex).menu.setPropertyParameters(["Continue", "Cycle", "Pingpong", "Off"]);
					effectsProperty.property(menuIndex).name = "Loop In";			
				}
		}
		
		// Function that returns the layer a property is on.
		function getLayer(property){
			if(property.propertyGroup() == null){
				return property;
			}
			else{
				return getLayer(property.propertyGroup());
			}
		}
		
		// Function that takes a name of an expression and returns the actual expression.
		function getExpression(expression){
			if(expression == "Bounce"){
				return '// BOUNCE\ne = effect("e")("Slider");\ng = effect("g")("Slider");\nnMax = effect("nMax")("Slider");\n\nn = 0;\n\nif (numKeys > 0){\n\t  n = nearestKey(time).index;\n\tif (key(n).time > time) n--;\n}\nif (n > 0){\n\tt = time - key(n).time;\n\tv = -velocityAtTime(key(n).time - .001)*e;\n\tvl = length(v);\n\tif (value instanceof Array){\n\t\tvu = (vl > 0) ? normalize(v) : [0,0,0];\n\t}else{\n\t\tvu = (v < 0) ? -1 : 1;\n\t}\n\ttCur = 0;\n\tsegDur = 2*vl/g;\n\ttNext = segDur;\n\tnb = 1; // number of bounces\n\twhile (tNext < t && nb <= nMax){\n\t\tvl *= e;\n\t\tsegDur *= e;\n\t\ttCur = tNext;\n\t\ttNext += segDur;\n\t\tnb++\n\t}\n\tif(nb <= nMax){\n\t\tdelta = t - tCur;\n\t\tvalue +  vu*delta*(vl - g*delta/2);\n\t}else{\n\t\tvalue;\n\t}\n}else{\n\tvalue;\n}';
			}
			else if(expression == "Overshoot"){
				return '// Inertial Bounce (moves settle into place after bouncing around a little)\namp =effect("Amp")("Slider");\nfreq =effect("Freq")("Slider");\ndecay =effect("Decay")("Slider");\n\nn =0;\nif(numKeys >0){\n\tn = nearestKey(time).index;\n\tif(key(n).time > time){\n\t\tn--;\n\t}\n}\n\nif(n ==0){\n\tt =0;\n}else{\n\tt = time - key(n).time;\n}\nif(n >0){\n\tv = velocityAtTime(key(n).time - thisComp.frameDuration/10);\n\tvalue + v*amp*Math.sin(freq*t*2*Math.PI)/Math.exp(decay*t);\n}else{\n\tvalue;\n}';
			}
			else if (expression == "Drift"){
				return 'if(value instanceof Array && value.length == 3){\n\tx = effect("Drift X")("Slider") * time;\n\ty = effect("Drift Y")("Slider") * time;\n\tz = effect("Drift Z")("Slider") * time;\n\tvalue + [x,y,z];\n}\nelse if(value instanceof Array && value.length == 2){\n\tx = effect("Drift X")("Slider") * time;\n\ty = effect("Drift Y")("Slider") * time;\n\tvalue + [x,y];\n}\nelse{\n\tx = effect("Drift")("Slider") * time;\n\tvalue + x;\n}\n';
			}
			else if (expression == "Loop Out"){
				return 'try{\n\tvar loop = effect("Loop Out")("Menu");\n\tif(loop == 1){\n\t\tloopOut("Continue");\n\t}\n\telse if(loop == 2){\n\t\tloopOut("Cycle");\n\t}\n\telse if(loop == 3){\n\t\tloopOut("Pingpong");\n\t}\n\telse{\n\t\tvalue;\n\t}\n}\ncatch(error){\n\tvalue;\n}';
			}
			else if (expression == "Loop In"){
				return 'try{\n\tvar loop = effect("Loop In")("Menu");\n\tif(loop == 1){\n\t\tloopIn("Continue");\n\t}\n\telse if(loop == 2){\n\t\tloopIn("Cycle");\n\t}\n\telse if(loop == 3){\n\t\tloopIn("Pingpong");\n\t}\n\telse{\n\t\tvalue;\n\t}\n}\ncatch(error){\n\tvalue;\n}';
			}
			else if (expression == "Loop"){
				return 'try{\n\tvar loop = effect("Loop Both")("Menu");\n\tif(loop == 1){\n\t\tif(time > key(numKeys).time){\n\t\t\tloopOut("Continue");\n\t\t}\n\t\telse{\n\t\t\tloopIn("Continue");\n\t\t}\n\t}\n\telse if(loop == 2){\n\t\tif(time > key(numKeys).time){\n\t\t\tloopOut("Cycle");\n\t\t}\n\t\telse{\n\t\t\tloopIn("Cycle");\n\t\t}\n\t}\n\telse if(loop == 3){\n\t\tif(time > key(numKeys).time){\n\t\t\tloopOut("Pingpong");\n\t\t}\n\t\telse{\n\t\t\tloopIn("Pingpong");\n\t\t}\n\t}\n\telse{\n\t\tvalue;\n\t}\n}\ncatch(error){\n\tvalue;\n}';
			}
			else if (expression == "Loop Both"){
				return 'try{\n\tvar loop = effect("Loop")("Menu");\n\tvar ls = effect("Loop Style")("Menu");\n\n\tvar loopStyle = "";\n\n\tif(ls == 1){\n\t\tloopStyle = "Continue";\n\t}\n\telse if(ls == 2){\n\t\tloopStyle = "Cycle";\n\t}\n\telse if(ls == 3){\n\t\tloopStyle = "Pingpong";\n\t}\n\telse{\n\t\tloopStyle = "Continue";\n\t}\n\n\tif(loop == 1){\n\t\tloopOut(loopStyle);\n\t}\n\telse if(loop == 2){\n\t\tloopIn(loopStyle);\n\t}\n\telse if(loop == 3){\n\t\tif(time > key(numKeys).time){\n\t\t\tloopOut(loopStyle);\n\t\t}\n\t\t\telse{\n\t\t\tloopIn(loopStyle);\n\t\t}\n\t}\n\telse{\n\t\tvalue;\n\t}\n}\ncatch(error){\n\tvalue;\n}';
			}
			else if (expression == "Link Focus Distance to Null"){
				return 'target = thisComp.layer("FOCUS_TARGET");\nv1 = target.toWorld(target.anchorPoint) - toWorld([0,0,0]);\nv2 = toWorldVec([0,0,1]);\ndot(v1,v2)';
			}
			else if (expression =="Link Scale to Cam Distance"){
				return 'C = thisComp.activeCamera;\nCP = fromWorld(C.toWorld([0,0,0]));\nL = thisLayer;\nLP = fromWorld(L.toWorld(anchorPoint));\nd = length(CP, LP);\nz = C.zoom;\nr = d/z\nr * value';
			}
			else if (expression == "Remove"){
				return "";
			}
			else{
				return "Nothing";
			}
		}
	  
		
	}

	////////////////////////////////////////////////////////////////////////	//////////////////
	// Window 01 - Batch Renamer Window
	// This script makes a window to batch change the names of things in the project panel.
	
	function batchRenamerWindow(){
	
		function buildWindow(){
			var windowName = "Batch Renamer";
			newWindow = new Window("palette", windowName, undefined, {resizeable:true, closeButton:true, floating:true});
		
			
			var tabRenamer = newWindow.add("group", undefined, "Batch Renamer");
			tabRenamer.resizeable = true;
			tabRenamer.orientation = "column";
			tabRenamer.alignment = ["fill", "top"];
			tabRenamer.preferredSize[0] = 300;
			
			//Renamer
			tabRenamer.alignChildren = ["fill", "top"];
			
			var groupRenamerOne = tabRenamer.add("group");
			groupRenamerOne.margins = [9,0,1,0];

			var groupRenamerTwo = tabRenamer.add("group");
			groupRenamerTwo.margins = [9,0,1,0];

			var groupRenamerThree = tabRenamer.add("group");
			groupRenamerThree.margins = [9,0,1,0];

			var groupRenamerFour = tabRenamer.add("group");
			groupRenamerFour.margins = [9,0,1,10];

			var groupRenamerFive = tabRenamer.add("group");

			var findWhat = groupRenamerOne.add("statictext", undefined, "Find what: ");
			findWhat.minimumSize = [70, 0];
			var find = groupRenamerOne.add("edittext", undefined, "");
			find.alignment = ["fill", "top"];

			var replaceWhat = groupRenamerTwo.add("statictext", undefined, "Replace with: ");
			replaceWhat.minimumSize = [70, 0];
			var replace = groupRenamerTwo.add("edittext", undefined, "");
			replace.alignment = ["fill", "top"];

			var prefixWhat = groupRenamerThree.add("statictext", undefined, "Prefix: ");
			prefixWhat.minimumSize = [70, 0];
			
			var prefix = groupRenamerThree.add("edittext", undefined, "");
			prefix.alignment = ["fill", "top"];

			var suffixWhat = groupRenamerFour.add("statictext", undefined, "Suffix: ");
			suffixWhat.minimumSize = [70, 0];
			var suffix = groupRenamerFour.add("edittext", undefined, "");
			suffix.alignment = ["fill", "top"];

			var renameButton = groupRenamerFive.add("button", undefined, "Rename");
			renameButton.preferredSize = [70, 25];
			renameButton.helpTip = "Batch renames all selected comps at once.";
			renameButton.alignment = ["center", "top"];
			renameButton.onClick = function(){
				var f = String(find.text);
				var r = String(replace.text);
				var p = String(prefix.text);
				var s = String(suffix.text);
				renamer(f, r, p, s);
			}
			
			return newWindow;
		}
	
		var newWindow = buildWindow();
		
		newWindow.onResizing = newWindow.onResize = function() {
			//this.children[0].children[1].children[0].orientation = "column";
			this.layout.resize();


		};
		if (newWindow != null && newWindow instanceof Window){
			newWindow.center();
			newWindow.show();
		}
		
		
	}
	
	
	////////////////////////////////////////////////////////////////////////	//////////////////
	// Window 02 - Batch Font Changer
	// This script makes a window to batch change all the fonts in the project.
	
	function batchFontChangerWindow(){
		var windowName = "Batch Font Changer";
		var fontArray = [];
			var length = app.project.items.length;
			
			//Searches through all items in a project panel, finds all the text layers, and adds their fonts to an array.
			for(var i = 1; i <= length; i++){
				var item = app.project.items[i];
				if(item instanceof CompItem){
					var numLayers = item.layers.length;

					//Searches through all layers in a comp.
					for(var j = 1; j <= numLayers; j++){
						var layer = item.layers[j];
						if (layer instanceof TextLayer){
								var fontAlreadyThere = false;
								for(var y = 0; y < fontArray.length; y++){
									if (layer.property("Source Text").value.font == fontArray[y]){
										fontAlreadyThere = true;
									}
								}
								if(!fontAlreadyThere){
									fontArray.push(layer.property("Source Text").value.font);
								}
								
						
						}
					}
				}
			}
			
		
		function buildWindow(){
			
			newWindow = new Window("palette", windowName, undefined, {resizeable:true, closeButton:true, floating:true});
			
			
			var tabTidy = newWindow.add("group", undefined, "Tidy Up");
			tabTidy.resizeable = true;
			tabTidy.orientation = "column";
			tabTidy.alignment = ["fill", "top"];
			
			
			tabTidy.alignChildren = ["fill", "top"];
			
			var groupRenamerOne = tabTidy.add("group");
			groupRenamerOne.margins = [9,0,1,0];

			var groupRenamerTwo = tabTidy.add("group");
			groupRenamerTwo.margins = [9,0,1,10];


			var groupRenamerFive = tabTidy.add("group");

			var findWhat = groupRenamerOne.add("statictext", undefined, "Find what: ");
			findWhat.minimumSize = [70, 0];
			var dropdown = groupRenamerOne.add("dropdownlist", undefined, "Dropdown");
			dropdown.minimumSize = [150, 20];
			dropdown.alignment = ["fill", "top"];

			var replaceWhat = groupRenamerTwo.add("statictext", undefined, "Replace with: ");
			replaceWhat.minimumSize = [70, 0];
			var dropdown2 = groupRenamerTwo.add("dropdownlist", undefined, "Dropdown");
			dropdown2.minimumSize = [150, 20];
			dropdown2.alignment = ["fill", "top"];
			
			
			var changeFontButton = groupRenamerFive.add("button", undefined, "Change Font");
			changeFontButton.preferredSize = [100, 25];
			changeFontButton.helpTip = "Batch renames all selected comps at once.";
			changeFontButton.alignment = ["center", "top"];
			changeFontButton.onClick = function(){
				replaceFonts(dropdown.selection, dropdown2.selection);
			}
			
			
			
			
					
			for(var i = 0; i < fontArray.length; i++){
				dropdown.add("item", fontArray[i]);
				dropdown2.add("item", fontArray[i]);
			}

			
			
			dropdown.selection = dropdown.items[0];
			dropdown2.selection = dropdown2.items[1];
			
			
			return newWindow;
		}
	
		if(fontArray.length <= 0){
			alert("There aren't any live fonts used in this project.", windowName);
		}
		else if(fontArray.length == 1){
			alert("There's only one font used in this project. To change it to a different font, create another text layer with the font you want, and then run this script again.", windowName);
		}
		else{
			var newWindow = buildWindow();
		
			newWindow.onResizing = newWindow.onResize = function() {
				//this.children[0].children[1].children[0].orientation = "column";
				this.layout.resize();


			};
			if (newWindow != null && newWindow instanceof Window){
				newWindow.center();
				newWindow.show();
			}
		}
		
		
		
	}
	
	////////////////////////////////////////////////////////////////////////	//////////////////
	// Window 03 - Batch Comp Settings Window
	// This script makes a window to batch change the settings of comps.
	
	function batchCompSettingsWindow(){
	
		function buildWindow(){
			var windowName = "Batch Comp Settings";
			newWindow = new Window("palette", windowName, undefined, {resizeable:true, closeButton:true, floating:true});
		
			
			var tabComps = newWindow.add("group", undefined, "Batch Comp Settings");
			tabComps.resizeable = true;
			tabComps.orientation = "column";
			tabComps.alignment = ["fill", "top"];
			tabComps.preferredSize[0] = 300;
			
			tabComps.alignChildren = ["fill", "top"];
			
			var groupCompsOne = tabComps.add("group");
			groupCompsOne.margins = [9,0,1,0];

			var groupCompsTwo = tabComps.add("group");
			groupCompsTwo.margins = [9,0,1,0];

			var groupCompsThree = tabComps.add("group");
			groupCompsThree.margins = [9,0,1,10];

			var groupCompsFour = tabComps.add("group");


			var compWidthWhat = groupCompsOne.add("statictext", undefined, "Width: ");
			compWidthWhat.minimumSize = [63, 0];
			var widthText = groupCompsOne.add("edittext", undefined, "");
			widthText.alignment = ["fill", "top"];

			var compHeightWhat = groupCompsTwo.add("statictext", undefined, "Height: ");
			compHeightWhat.minimumSize = [63, 0];
			var heightText = groupCompsTwo.add("edittext", undefined, "");
			heightText.alignment = ["fill", "top"];

			var frameRateWhat = groupCompsThree.add("statictext", undefined, "Frame Rate: ");
			frameRateWhat.minimumSize = [63, 0];
			var frameRateText = groupCompsThree.add("edittext", undefined, "");
			frameRateText.alignment = ["fill", "top"];

			var compsButton = groupCompsFour.add("button", undefined, "Change Comp Settings");
			compsButton.preferredSize = [140, 25];
			compsButton.helpTip = "Changes the comp settings for all selected comps at once.";
			compsButton.alignment = ["center", "top"];
			compsButton.onClick = function() {
				var w = String(widthText.text);
				var h = String(heightText.text);
				var fr = String(frameRateText.text);
				changeComps(w, h, fr);
			}
			
			return newWindow;
		}
	
		var newWindow = buildWindow();
		
		newWindow.onResizing = newWindow.onResize = function() {
			//this.children[0].children[1].children[0].orientation = "column";
			this.layout.resize();


		};
		if (newWindow != null && newWindow instanceof Window){
			newWindow.center();
			newWindow.show();
		}
		
		
	}
	
	
}